class ElementalPower(object):
    def __init__(self, element: str, attack: int, dmg: int, res: int):
        self.element = element
        self.attack = attack
        self.dmg = dmg
        self.res = res
